import sys
import typing

GenericType = typing.TypeVar("GenericType")


class RawBlendFileReader:
    pass


def main():
    ''' 

    '''

    pass


def read_blend_rend_chunk(filepath):
    ''' 

    '''

    pass
